/*
 * Fooyin
 * Copyright © 2026, Luke Taylor <luket@pm.me>
 *
 * Fooyin is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fooyin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fooyin.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include "fycore_export.h"

#include <core/track.h>

#include <optional>

namespace Fooyin {
class FYCORE_EXPORT PendingTrackCoverProvider
{
public:
    virtual ~PendingTrackCoverProvider() = default;

    /*!
     * Returns a pending embedded cover override for @p track and @p type.
     * @note A returned CoverImage with empty data means a pending removal; std::nullopt means no pending override.
     */
    [[nodiscard]] virtual std::optional<CoverImage> pendingTrackCover(const Track& track, Track::Cover type) const = 0;
};
} // namespace Fooyin
