/****************************************************************************
** Meta object code from reading C++ file 'viewmenu.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../fooyin/src/gui/menubar/viewmenu.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'viewmenu.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Fooyin8ViewMenuE_t {};
} // unnamed namespace

template <> constexpr inline auto Fooyin::ViewMenu::qt_create_metaobjectdata<qt_meta_tag_ZN6Fooyin8ViewMenuE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Fooyin::ViewMenu",
        "focusSearchBar",
        "",
        "openQuickSetup",
        "openPlaybackQueue",
        "openPlaylistManager",
        "openLog",
        "openScriptEditor",
        "showNowPlaying"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'focusSearchBar'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openQuickSetup'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openPlaybackQueue'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openPlaylistManager'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openLog'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openScriptEditor'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showNowPlaying'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ViewMenu, qt_meta_tag_ZN6Fooyin8ViewMenuE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Fooyin::ViewMenu::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin8ViewMenuE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin8ViewMenuE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Fooyin8ViewMenuE_t>.metaTypes,
    nullptr
} };

void Fooyin::ViewMenu::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ViewMenu *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->focusSearchBar(); break;
        case 1: _t->openQuickSetup(); break;
        case 2: _t->openPlaybackQueue(); break;
        case 3: _t->openPlaylistManager(); break;
        case 4: _t->openLog(); break;
        case 5: _t->openScriptEditor(); break;
        case 6: _t->showNowPlaying(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ViewMenu::*)()>(_a, &ViewMenu::focusSearchBar, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewMenu::*)()>(_a, &ViewMenu::openQuickSetup, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewMenu::*)()>(_a, &ViewMenu::openPlaybackQueue, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewMenu::*)()>(_a, &ViewMenu::openPlaylistManager, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewMenu::*)()>(_a, &ViewMenu::openLog, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewMenu::*)()>(_a, &ViewMenu::openScriptEditor, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewMenu::*)()>(_a, &ViewMenu::showNowPlaying, 6))
            return;
    }
}

const QMetaObject *Fooyin::ViewMenu::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Fooyin::ViewMenu::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin8ViewMenuE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Fooyin::ViewMenu::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void Fooyin::ViewMenu::focusSearchBar()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Fooyin::ViewMenu::openQuickSetup()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Fooyin::ViewMenu::openPlaybackQueue()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Fooyin::ViewMenu::openPlaylistManager()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Fooyin::ViewMenu::openLog()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Fooyin::ViewMenu::openScriptEditor()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Fooyin::ViewMenu::showNowPlaying()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
