/*
 * Fooyin
 * Copyright © 2026, Luke Taylor <luket@pm.me>
 *
 * Fooyin is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fooyin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fooyin.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include "fycore_export.h"

#include <QString>

namespace Fooyin {
namespace PlaycountSettings {
constexpr auto ReadTag      = "Playcount/ReadTag";
constexpr auto WriteTag     = "Playcount/WriteTag";
constexpr auto ReadId3Popm  = "Playcount/ReadId3Popm";
constexpr auto WriteId3Popm = "Playcount/WriteId3Popm";

constexpr auto DefaultAutomatic    = "Automatic";
constexpr auto DefaultFmpsTag      = "FMPS_PLAYCOUNT";
constexpr auto DefaultReadId3Popm  = true;
constexpr auto DefaultWriteId3Popm = true;
} // namespace PlaycountSettings

struct FYCORE_EXPORT PlaycountTagPolicy
{
    QString readTag;
    QString writeTag;
    bool readId3Popm{true};
    bool writeId3Popm{true};

    [[nodiscard]] bool automaticRead() const;
    [[nodiscard]] bool shouldReadTag(const QString& tag, bool currentPlaycountSet) const;
    [[nodiscard]] QString effectiveWriteTag() const;
};

[[nodiscard]] FYCORE_EXPORT PlaycountTagPolicy playcountTagPolicy();
} // namespace Fooyin
