/****************************************************************************
** Meta object code from reading C++ file 'libraryscanner.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../fooyin/src/core/library/libraryscanner.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'libraryscanner.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Fooyin14LibraryScannerE_t {};
} // unnamed namespace

template <> constexpr inline auto Fooyin::LibraryScanner::qt_create_metaobjectdata<qt_meta_tag_ZN6Fooyin14LibraryScannerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Fooyin::LibraryScanner",
        "progressChanged",
        "",
        "Fooyin::ScanProgress",
        "progress",
        "statusChanged",
        "Fooyin::LibraryInfo",
        "library",
        "scanUpdate",
        "Fooyin::ScanResult",
        "result",
        "scannedTracks",
        "Fooyin::TrackList",
        "tracks",
        "playlistLoaded",
        "scanLibrary",
        "onlyModified",
        "Fooyin::LibraryScanConfig",
        "config",
        "scanLibraryDirectoies",
        "dirs",
        "scanTracks",
        "scanFiles",
        "libraryTracks",
        "QList<QUrl>",
        "urls",
        "scanPlaylist"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'progressChanged'
        QtMocHelpers::SignalData<void(const Fooyin::ScanProgress &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void(const Fooyin::LibraryInfo &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'scanUpdate'
        QtMocHelpers::SignalData<void(const Fooyin::ScanResult &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'scannedTracks'
        QtMocHelpers::SignalData<void(const Fooyin::TrackList &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'playlistLoaded'
        QtMocHelpers::SignalData<void(const Fooyin::TrackList &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'scanLibrary'
        QtMocHelpers::SlotData<void(const Fooyin::LibraryInfo &, const Fooyin::TrackList &, bool, const Fooyin::LibraryScanConfig &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 12, 13 }, { QMetaType::Bool, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'scanLibraryDirectoies'
        QtMocHelpers::SlotData<void(const Fooyin::LibraryInfo &, const QStringList &, const Fooyin::TrackList &, const Fooyin::LibraryScanConfig &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::QStringList, 20 }, { 0x80000000 | 12, 13 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'scanTracks'
        QtMocHelpers::SlotData<void(const Fooyin::TrackList &, bool, const Fooyin::LibraryScanConfig &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { QMetaType::Bool, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'scanFiles'
        QtMocHelpers::SlotData<void(const Fooyin::TrackList &, const QList<QUrl> &, const Fooyin::LibraryScanConfig &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 23 }, { 0x80000000 | 24, 25 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'scanPlaylist'
        QtMocHelpers::SlotData<void(const Fooyin::TrackList &, const QList<QUrl> &, const Fooyin::LibraryScanConfig &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 23 }, { 0x80000000 | 24, 25 }, { 0x80000000 | 17, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LibraryScanner, qt_meta_tag_ZN6Fooyin14LibraryScannerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Fooyin::LibraryScanner::staticMetaObject = { {
    QMetaObject::SuperData::link<Worker::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin14LibraryScannerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin14LibraryScannerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Fooyin14LibraryScannerE_t>.metaTypes,
    nullptr
} };

void Fooyin::LibraryScanner::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LibraryScanner *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->progressChanged((*reinterpret_cast<std::add_pointer_t<Fooyin::ScanProgress>>(_a[1]))); break;
        case 1: _t->statusChanged((*reinterpret_cast<std::add_pointer_t<Fooyin::LibraryInfo>>(_a[1]))); break;
        case 2: _t->scanUpdate((*reinterpret_cast<std::add_pointer_t<Fooyin::ScanResult>>(_a[1]))); break;
        case 3: _t->scannedTracks((*reinterpret_cast<std::add_pointer_t<Fooyin::TrackList>>(_a[1]))); break;
        case 4: _t->playlistLoaded((*reinterpret_cast<std::add_pointer_t<Fooyin::TrackList>>(_a[1]))); break;
        case 5: _t->scanLibrary((*reinterpret_cast<std::add_pointer_t<Fooyin::LibraryInfo>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Fooyin::TrackList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Fooyin::LibraryScanConfig>>(_a[4]))); break;
        case 6: _t->scanLibraryDirectoies((*reinterpret_cast<std::add_pointer_t<Fooyin::LibraryInfo>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Fooyin::TrackList>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Fooyin::LibraryScanConfig>>(_a[4]))); break;
        case 7: _t->scanTracks((*reinterpret_cast<std::add_pointer_t<Fooyin::TrackList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Fooyin::LibraryScanConfig>>(_a[3]))); break;
        case 8: _t->scanFiles((*reinterpret_cast<std::add_pointer_t<Fooyin::TrackList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Fooyin::LibraryScanConfig>>(_a[3]))); break;
        case 9: _t->scanPlaylist((*reinterpret_cast<std::add_pointer_t<Fooyin::TrackList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Fooyin::LibraryScanConfig>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Fooyin::ScanProgress >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Fooyin::TrackList >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Fooyin::TrackList >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Fooyin::TrackList >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Fooyin::TrackList >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Fooyin::TrackList >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Fooyin::TrackList >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Fooyin::TrackList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LibraryScanner::*)(const Fooyin::ScanProgress & )>(_a, &LibraryScanner::progressChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryScanner::*)(const Fooyin::LibraryInfo & )>(_a, &LibraryScanner::statusChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryScanner::*)(const Fooyin::ScanResult & )>(_a, &LibraryScanner::scanUpdate, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryScanner::*)(const Fooyin::TrackList & )>(_a, &LibraryScanner::scannedTracks, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryScanner::*)(const Fooyin::TrackList & )>(_a, &LibraryScanner::playlistLoaded, 4))
            return;
    }
}

const QMetaObject *Fooyin::LibraryScanner::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Fooyin::LibraryScanner::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin14LibraryScannerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "LibraryScanHost"))
        return static_cast< LibraryScanHost*>(this);
    return Worker::qt_metacast(_clname);
}

int Fooyin::LibraryScanner::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Worker::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void Fooyin::LibraryScanner::progressChanged(const Fooyin::ScanProgress & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Fooyin::LibraryScanner::statusChanged(const Fooyin::LibraryInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Fooyin::LibraryScanner::scanUpdate(const Fooyin::ScanResult & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Fooyin::LibraryScanner::scannedTracks(const Fooyin::TrackList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Fooyin::LibraryScanner::playlistLoaded(const Fooyin::TrackList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
