/****************************************************************************
** Meta object code from reading C++ file 'lyricsview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../fooyin/src/plugins/lyrics/lyricsview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lyricsview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Fooyin6Lyrics10LyricsViewE_t {};
} // unnamed namespace

template <> constexpr inline auto Fooyin::Lyrics::LyricsView::qt_create_metaobjectdata<qt_meta_tag_ZN6Fooyin6Lyrics10LyricsViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Fooyin::Lyrics::LyricsView",
        "dragSeekingChanged",
        "",
        "active",
        "lineClicked",
        "QModelIndex",
        "index",
        "QPoint",
        "pos",
        "lineDragSeekRequested",
        "userScrolling",
        "viewportResized"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dragSeekingChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'lineClicked'
        QtMocHelpers::SignalData<void(const QModelIndex &, const QPoint &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'lineDragSeekRequested'
        QtMocHelpers::SignalData<void(const QModelIndex &, const QPoint &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'userScrolling'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewportResized'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LyricsView, qt_meta_tag_ZN6Fooyin6Lyrics10LyricsViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Fooyin::Lyrics::LyricsView::staticMetaObject = { {
    QMetaObject::SuperData::link<QListView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin6Lyrics10LyricsViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin6Lyrics10LyricsViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Fooyin6Lyrics10LyricsViewE_t>.metaTypes,
    nullptr
} };

void Fooyin::Lyrics::LyricsView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LyricsView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dragSeekingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->lineClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 2: _t->lineDragSeekRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 3: _t->userScrolling(); break;
        case 4: _t->viewportResized(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LyricsView::*)(bool )>(_a, &LyricsView::dragSeekingChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LyricsView::*)(const QModelIndex & , const QPoint & )>(_a, &LyricsView::lineClicked, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LyricsView::*)(const QModelIndex & , const QPoint & )>(_a, &LyricsView::lineDragSeekRequested, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LyricsView::*)()>(_a, &LyricsView::userScrolling, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (LyricsView::*)()>(_a, &LyricsView::viewportResized, 4))
            return;
    }
}

const QMetaObject *Fooyin::Lyrics::LyricsView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Fooyin::Lyrics::LyricsView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin6Lyrics10LyricsViewE_t>.strings))
        return static_cast<void*>(this);
    return QListView::qt_metacast(_clname);
}

int Fooyin::Lyrics::LyricsView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void Fooyin::Lyrics::LyricsView::dragSeekingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Fooyin::Lyrics::LyricsView::lineClicked(const QModelIndex & _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void Fooyin::Lyrics::LyricsView::lineDragSeekRequested(const QModelIndex & _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void Fooyin::Lyrics::LyricsView::userScrolling()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Fooyin::Lyrics::LyricsView::viewportResized()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
