/****************************************************************************
** Meta object code from reading C++ file 'dspchainstore.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../fooyin/src/core/engine/dsp/dspchainstore.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dspchainstore.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Fooyin13DspChainStoreE_t {};
} // unnamed namespace

template <> constexpr inline auto Fooyin::DspChainStore::qt_create_metaobjectdata<qt_meta_tag_ZN6Fooyin13DspChainStoreE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Fooyin::DspChainStore",
        "activeChainChanged",
        "",
        "Fooyin::Engine::DspChains",
        "chain",
        "liveDspSettingsChanged",
        "Fooyin::Engine::DspChainScope",
        "scope",
        "uint64_t",
        "instanceId",
        "settings",
        "persisted",
        "source",
        "dspEnabledChanged",
        "enabled"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'activeChainChanged'
        QtMocHelpers::SignalData<void(const Fooyin::Engine::DspChains &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'liveDspSettingsChanged'
        QtMocHelpers::SignalData<void(Fooyin::Engine::DspChainScope, uint64_t, const QByteArray &, bool, QObject *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 }, { QMetaType::QByteArray, 10 }, { QMetaType::Bool, 11 },
            { QMetaType::QObjectStar, 12 },
        }}),
        // Signal 'dspEnabledChanged'
        QtMocHelpers::SignalData<void(Fooyin::Engine::DspChainScope, uint64_t, bool, QObject *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 }, { QMetaType::Bool, 14 }, { QMetaType::QObjectStar, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DspChainStore, qt_meta_tag_ZN6Fooyin13DspChainStoreE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Fooyin::DspChainStore::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin13DspChainStoreE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin13DspChainStoreE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Fooyin13DspChainStoreE_t>.metaTypes,
    nullptr
} };

void Fooyin::DspChainStore::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DspChainStore *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activeChainChanged((*reinterpret_cast<std::add_pointer_t<Fooyin::Engine::DspChains>>(_a[1]))); break;
        case 1: _t->liveDspSettingsChanged((*reinterpret_cast<std::add_pointer_t<Fooyin::Engine::DspChainScope>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint64_t>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[5]))); break;
        case 2: _t->dspEnabledChanged((*reinterpret_cast<std::add_pointer_t<Fooyin::Engine::DspChainScope>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint64_t>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[4]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DspChainStore::*)(const Fooyin::Engine::DspChains & )>(_a, &DspChainStore::activeChainChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DspChainStore::*)(Fooyin::Engine::DspChainScope , uint64_t , const QByteArray & , bool , QObject * )>(_a, &DspChainStore::liveDspSettingsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DspChainStore::*)(Fooyin::Engine::DspChainScope , uint64_t , bool , QObject * )>(_a, &DspChainStore::dspEnabledChanged, 2))
            return;
    }
}

const QMetaObject *Fooyin::DspChainStore::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Fooyin::DspChainStore::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin13DspChainStoreE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Fooyin::DspChainStore::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void Fooyin::DspChainStore::activeChainChanged(const Fooyin::Engine::DspChains & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Fooyin::DspChainStore::liveDspSettingsChanged(Fooyin::Engine::DspChainScope _t1, uint64_t _t2, const QByteArray & _t3, bool _t4, QObject * _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 2
void Fooyin::DspChainStore::dspEnabledChanged(Fooyin::Engine::DspChainScope _t1, uint64_t _t2, bool _t3, QObject * _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3, _t4);
}
QT_WARNING_POP
