/****************************************************************************
** Meta object code from reading C++ file 'artworkfinder.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../fooyin/src/gui/artwork/artworkfinder.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'artworkfinder.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Fooyin13ArtworkFinderE_t {};
} // unnamed namespace

template <> constexpr inline auto Fooyin::ArtworkFinder::qt_create_metaobjectdata<qt_meta_tag_ZN6Fooyin13ArtworkFinderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Fooyin::ArtworkFinder",
        "coverFound",
        "",
        "Fooyin::SearchResult",
        "result",
        "coverLoaded",
        "QUrl",
        "url",
        "Fooyin::ArtworkResult",
        "coverLoadProgress",
        "progress",
        "coverLoadError",
        "searchFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'coverFound'
        QtMocHelpers::SignalData<void(const Fooyin::SearchResult &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'coverLoaded'
        QtMocHelpers::SignalData<void(const QUrl &, const Fooyin::ArtworkResult &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 4 },
        }}),
        // Signal 'coverLoadProgress'
        QtMocHelpers::SignalData<void(const QUrl &, int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::Int, 10 },
        }}),
        // Signal 'coverLoadError'
        QtMocHelpers::SignalData<void(const QUrl &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'searchFinished'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ArtworkFinder, qt_meta_tag_ZN6Fooyin13ArtworkFinderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Fooyin::ArtworkFinder::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin13ArtworkFinderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin13ArtworkFinderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Fooyin13ArtworkFinderE_t>.metaTypes,
    nullptr
} };

void Fooyin::ArtworkFinder::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ArtworkFinder *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->coverFound((*reinterpret_cast<std::add_pointer_t<Fooyin::SearchResult>>(_a[1]))); break;
        case 1: _t->coverLoaded((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Fooyin::ArtworkResult>>(_a[2]))); break;
        case 2: _t->coverLoadProgress((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->coverLoadError((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 4: _t->searchFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ArtworkFinder::*)(const Fooyin::SearchResult & )>(_a, &ArtworkFinder::coverFound, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArtworkFinder::*)(const QUrl & , const Fooyin::ArtworkResult & )>(_a, &ArtworkFinder::coverLoaded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArtworkFinder::*)(const QUrl & , int )>(_a, &ArtworkFinder::coverLoadProgress, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArtworkFinder::*)(const QUrl & )>(_a, &ArtworkFinder::coverLoadError, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArtworkFinder::*)()>(_a, &ArtworkFinder::searchFinished, 4))
            return;
    }
}

const QMetaObject *Fooyin::ArtworkFinder::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Fooyin::ArtworkFinder::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin13ArtworkFinderE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Fooyin::ArtworkFinder::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void Fooyin::ArtworkFinder::coverFound(const Fooyin::SearchResult & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Fooyin::ArtworkFinder::coverLoaded(const QUrl & _t1, const Fooyin::ArtworkResult & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void Fooyin::ArtworkFinder::coverLoadProgress(const QUrl & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void Fooyin::ArtworkFinder::coverLoadError(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Fooyin::ArtworkFinder::searchFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
