/*
 * Fooyin
 * Copyright © 2026, Luke Taylor <luket@pm.me>
 *
 * Fooyin is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fooyin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fooyin.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <mutex>
#include <optional>

namespace Fooyin::Sdl {
class SdlAudioSubsystem;

class SdlAudioLease
{
public:
    ~SdlAudioLease();

    SdlAudioLease(const SdlAudioLease&)            = delete;
    SdlAudioLease& operator=(const SdlAudioLease&) = delete;

    SdlAudioLease(SdlAudioLease&& other) noexcept;
    SdlAudioLease& operator=(SdlAudioLease&& other) noexcept;

private:
    friend class SdlAudioSubsystem;

    explicit SdlAudioLease(SdlAudioSubsystem* subsystem);

    SdlAudioSubsystem* m_subsystem;
};

class SdlAudioSubsystem
{
public:
    [[nodiscard]] std::optional<SdlAudioLease> acquire();

private:
    friend class SdlAudioLease;

    void release();

    std::mutex m_mutex;
};
} // namespace Fooyin::Sdl
