/****************************************************************************
** Meta object code from reading C++ file 'nowplayingoutputservice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../fooyin/src/gui/nowplayingoutput/nowplayingoutputservice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'nowplayingoutputservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Fooyin23NowPlayingOutputServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto Fooyin::NowPlayingOutputService::qt_create_metaobjectdata<qt_meta_tag_ZN6Fooyin23NowPlayingOutputServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Fooyin::NowPlayingOutputService",
        "UpdateEvents",
        "UpdateEvent",
        "UpdateNone",
        "UpdateTrack",
        "UpdatePlayPause",
        "UpdateStop",
        "UpdateSecond",
        "UpdatePlayed",
        "DefaultEvents",
        "OutputTargets",
        "OutputTarget",
        "NoOutput",
        "OutputClipboard",
        "OutputFile",
        "OutputOptions",
        "OutputOption",
        "NoOptions",
        "OutputAppendFile",
        "OutputDefaultOptions"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'UpdateEvents'
        QtMocHelpers::EnumData<UpdateEvents>(1, 2, QMC::EnumIsFlag).add({
            {    3, UpdateEvent::UpdateNone },
            {    4, UpdateEvent::UpdateTrack },
            {    5, UpdateEvent::UpdatePlayPause },
            {    6, UpdateEvent::UpdateStop },
            {    7, UpdateEvent::UpdateSecond },
            {    8, UpdateEvent::UpdatePlayed },
            {    9, UpdateEvent::DefaultEvents },
        }),
        // flag 'OutputTargets'
        QtMocHelpers::EnumData<OutputTargets>(10, 11, QMC::EnumIsFlag).add({
            {   12, OutputTarget::NoOutput },
            {   13, OutputTarget::OutputClipboard },
            {   14, OutputTarget::OutputFile },
        }),
        // flag 'OutputOptions'
        QtMocHelpers::EnumData<OutputOptions>(15, 16, QMC::EnumIsFlag).add({
            {   17, OutputOption::NoOptions },
            {   18, OutputOption::OutputAppendFile },
            {   19, OutputOption::OutputDefaultOptions },
        }),
    };
    return QtMocHelpers::metaObjectData<NowPlayingOutputService, qt_meta_tag_ZN6Fooyin23NowPlayingOutputServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Fooyin::NowPlayingOutputService::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin23NowPlayingOutputServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin23NowPlayingOutputServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Fooyin23NowPlayingOutputServiceE_t>.metaTypes,
    nullptr
} };

void Fooyin::NowPlayingOutputService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NowPlayingOutputService *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Fooyin::NowPlayingOutputService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Fooyin::NowPlayingOutputService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Fooyin23NowPlayingOutputServiceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Fooyin::NowPlayingOutputService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
